/* 
 * Thread that accepts the SIGINT signal using sigwaitinfo. Displays a
 * message when received. Terminates the process when the fourth
 * signal is received.
 */
#include <stdio.h>
#include <signal.h>
#include <pthread.h>
#include <unistd.h>
#include <string.h>
static int error_status=-1;

void * sigwaiter (void * arg)
{
  sigset_t set;
  siginfo_t siginfo;
  int counter = 0;

  sigemptyset (&set);
  sigaddset (&set, SIGINT);
  /* The signal mask is inherited from the parent */
  /* It must have SIGINT masked */

  while (1) {
    if (sigwaitinfo (&set, &siginfo)) {
      /*sigwaitinfo error*/
      perror ("sigwaitinfo");
      pthread_exit ((void*)&error_status);
    }
    if (siginfo.si_signo == SIGINT) {
      counter ++;
      if (counter < 4) {
	printf ("SIGINT %d received. signo: %d, si_code:%d, si_value:%d\n", 
		counter, siginfo.si_signo, siginfo.si_code, 
		siginfo.si_value.sival_int);
      } else {
	printf ("Final signal received\n");
	exit(0);
      }
    } else {
      /* unexpected signal */
      pthread_exit ((void *)&error_status);
    }
  }
}


/* Main thread, that sets the mask for itself and its child */

int main ()
{
  sigset_t set;
  pthread_t th;
  union sigval value = {0};
  struct sigaction action;
  int ret;

  /* Set signal mask */
  sigemptyset(&set);
  sigaddset(&set, SIGINT);
  if ((ret = pthread_sigmask(SIG_BLOCK, &set, NULL)))
    printf ("pthread_sigmask:%s\n", strerror (ret));

  /* Set SA_SIGINFO for SIGINT */
  /* This is unnecessary in MaRTE OS since all the signals behave as
     realtime signals by default, but it is necessary to write a full
     conforming POSIX application. */
  action.sa_handler = SIG_DFL;
  sigemptyset(&action.sa_mask);
  action.sa_flags = SA_SIGINFO;
  if (sigaction (SIGINT, &action, NULL))
    perror ("sigaction");

  /* Create task */
  if ((ret = pthread_create(&th, NULL, sigwaiter, NULL)))
    printf ("Error in thread creation:%s\n", strerror (ret));

  /* Send signals */
  while (1) {
    value.sival_int++;
    printf ("Main sends signal %d\n", value.sival_int);
    if (sigqueue (0, SIGINT, value))
      perror ("sigqueue");
    sleep(10);
  }
}
