
#ifndef __MARTE_TLSF_MAPPING_H__
#define __MARTE_TLSF_MAPPING_H__ 1
#ifdef  USE_TLSF_MEMORY_ALLOCATOR

/*
 * log2size ()  returns cell  of log2 (len)  it does a  search between
 * MIN_SIZE and MAX_SIZE values in order to find the log2 of the size.
 * Theorically  we have obtained  that this  method is  more efficient
 * than the asm instruction which does the same operation
 */

static inline __s32 log2size (size_t size, size_t *new_size) {
  
  __s32 i;

  /* Other way quicker and also more machine dependent that the first one */
  /*------------------------------*/
  // << MaRTE
  // i = TLSF_fls(size);
  bit_scan_reverse (size, &i);
  // MaRTE >>
  
    
  *new_size = (1 << i);

  return i;
}

/*
 * mapping_function () returns first and second level index
 *
 * first level index function is:
 * fl = log2 (size)
 *
 * and second level index function is:
 * sl = (size - 2**(log2size (size))) / (log2 (size) - log2 (MAX_SL_INDEX))
 *
 */

static inline __s32 mapping_function (size_t size, __s32 *fl, __s32 *sl,
				      TLSF_t *ptr_TLSF){
  __s32 aux, new_len;
   
  /* This is a new way to calculate first level index 
     and second level index, it is quicker than older one */
  
  *fl = log2size (size, &new_len);
  aux = (*fl - ptr_TLSF -> max_sl_log2_index);
  *sl = ((size ^ new_len) >> aux);
  
  return 0;
}

#endif // USE_TLSF_MEMORY_ALLOCATOR
#endif // __MARTE_TLSF_MAPPING_H__
