------------------------------------------------------------------------------
-- --------------------         M a R T E   O S         ------------------- --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--    'K e r n e l . A p p l i c a t i o n _ S c h e d u l i n g _ D a t a'
--
--                                   Body
--
--
--
--  File 'k-appsched_data.adb'                                         By MAR.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------
with Configuration_Parameters;
with Singly_Linked_Lists.Resources;
pragma Elaborate_All (Singly_Linked_Lists.Resources);

package body Kernel.Application_Scheduling_Data is

   use AppSched_Events_Lists;
   use Task_Containers_Lists;
   use AppScheduler_Data_Lists;

   package CP renames Configuration_Parameters;

   ----------------------------------------------------------------------------
   -- Application scheduling events pool --------------------------------------
   ----------------------------------------------------------------------------

   Free_AppSched_Events : array (1 .. CP.Num_Application_Scheduling_Events_Mx)
     of aliased AppSched_Events_Lists.Element;
   package AppSched_Events_Pool_Mangement is
     new AppSched_Events_Lists.Resources;

   ----------------------------
   -- Release_AppSched_Event --
   ----------------------------
   procedure Release_AppSched_Event (E : in AppSched_Event_Ac)
     renames AppSched_Events_Pool_Mangement.Release;

   ----------------------------
   -- Request_AppSched_Event --
   ----------------------------
   function Request_AppSched_Event return AppSched_Event_Ac
     renames AppSched_Events_Pool_Mangement.Request;


   ----------------------------------------------------------------------------
   -- Task Containers ">" -----------------------------------------------------
   ----------------------------------------------------------------------------
   function ">" (Left, Right : in Task_Container_Ac) return Boolean is
   begin
      return Left.T.Active_Prio > Right.T.Active_Prio;
   end ">";

   ----------------------------------------------------------------------------
   -- Task Containers pool ----------------------------------------------------
   ----------------------------------------------------------------------------

   ----------
   -- Pool --
   ----------
   Free_TCs : array (1 .. CP.Num_Application_Scheduled_Tasks_Mx)
     of aliased Task_Containers_Lists.Element;
   package TCs_Pool_Mangement is new Task_Containers_Lists.Resources;

   ----------------------------
   -- Release_Task_Container --
   ----------------------------
   procedure Release_Task_Container (TC : in Task_Container_Ac)
     renames TCs_Pool_Mangement.Release;

   ----------------------------
   -- Request_Task_Container --
   ----------------------------
   function Request_Task_Container return Task_Container_Ac
     renames TCs_Pool_Mangement.Request;

   -----------------------------------
   -- Task_Container_Pool_Exhausted --
   -----------------------------------
   function Any_Task_Container_Free_In_Pool return Boolean
     renames TCs_Pool_Mangement.Resources_Left;

   ----------------------------------------------------------------------------
   -- 'AppScheduler_Data' pool ------------------------------------------------
   ----------------------------------------------------------------------------

   ------------------------------
   -- Null 'AppScheduler_Data' --
   ------------------------------
   AppScheduler_Data_Null : aliased constant AppScheduler_Data :=
     (Tasks_Q                   => Task_Containers_Lists.Null_List,
      Events_Q                  => AppSched_Events_Lists.Null_List,
      Flags                     => Option_Sets.Empty_Option_Set,
      Events_Mask               => Option_Sets.Empty_Option_Set,
      Waiting_For_Signals       => False,
      Clock                     => Timers.CLOCK_REALTIME,
      Reserved_TCs              => Task_Containers_Lists.Null_List,
      Suspension_Deferred_Tasks => Sched_App_Data_Lists.Null_List,
      Reply                     => (others => 0),
      Reply_Size                => 0);

   ----------
   -- Pool --
   ----------
   Free_AppScheduler_Data : array (1 .. CP.Num_Application_Schedulers_Mx)
     of aliased AppScheduler_Data_Lists.Element;
   package AppScheduler_Data_Pool_Mangement is
     new AppScheduler_Data_Lists.Resources;

   ---------------------
   -- 'To_Option_Set' --
   ---------------------
   --
   --  Transform an 'Event_Code_T' into an 'Option_Set_T' that only contains
   --  the option corresponding to the 'Event_Code_T'
   function To_Option_Set (Event : in Event_Code_T)
                           return Option_Sets.Option_Set is
      function To_Int is new Ada.Unchecked_Conversion (Event_Code_T,
                                                       Integer);
   begin
      return Option_Sets.To_Option_Number (To_Int (Event));
   end To_Option_Set;

   -------------------------------
   -- Release_AppScheduler_Data --
   -------------------------------
   procedure Release_AppScheduler_Data
     (AppScheduler_Data : in AppScheduler_Data_Ac) is
      TC : Task_Container_Ac := Head (AppScheduler_Data.Tasks_Q);
      Event_Ac : AppSched_Event_Ac := Head (AppScheduler_Data.Events_Q);
   begin
      --  Release TCs in 'Tasks_Q'
      while TC /= null loop
         Dequeue_Head (AppScheduler_Data.Tasks_Q);
         Release_Task_Container (TC);
         TC := Head (AppScheduler_Data.Tasks_Q);
      end loop;
      --  Release Events in 'Events_Q'
      while Event_Ac /= null loop
         Dequeue_Head (AppScheduler_Data.Events_Q);
         Release_AppSched_Event (Event_Ac);
         Event_Ac := Head (AppScheduler_Data.Events_Q);
      end loop;

      AppScheduler_Data_Pool_Mangement.Release (AppScheduler_Data_Null'Access,
                                                AppScheduler_Data);
   end Release_AppScheduler_Data;

   -------------------------------
   -- Request_AppScheduler_Data --
   -------------------------------
   function Request_AppScheduler_Data return AppScheduler_Data_Ac
     renames AppScheduler_Data_Pool_Mangement.Request;

   ----------------------------------------------------------------------------
   -- 'Sched_App_Data' pool ---------------------------------------------------
   ----------------------------------------------------------------------------
   Free_Sched_App_Data : array (1 .. CP.Num_Of_Sched_App_Data)
     of aliased Sched_App_Data_Lists.Element;
   Sched_App_Data_Counter : Integer range
     Free_Sched_App_Data'First - 1 .. Free_Sched_App_Data'Last :=
     Free_Sched_App_Data'First - 1;

   function Request_Sched_App_Data return Sched_App_Data_Ac is
   begin
      Sched_App_Data_Counter := Sched_App_Data_Counter + 1;
      return Free_Sched_App_Data (Sched_App_Data_Counter)'Access;
   end Request_Sched_App_Data;


begin
   --  Initialize 'AppSched_Events' pool
   for I in Free_AppSched_Events'Range loop
      AppSched_Events_Pool_Mangement.Release (Free_AppSched_Events (I)'Access);
   end loop;

   --  Initialize TCs pool
   for I in Free_TCs'Range loop
      TCs_Pool_Mangement.Release (Free_TCs (I)'Access);
   end loop;

   --  Initialize 'AppScheduler_Data' pool
   for I in Free_AppScheduler_Data'Range loop
      AppScheduler_Data_Pool_Mangement.Release
        (AppScheduler_Data_Null'Access,
         Free_AppScheduler_Data (I)'Access);
   end loop;

end Kernel.Application_Scheduling_Data;
