------------------------------------------------------------------------------
-- --------------------         M a R T E   O S         ------------------- --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--    'K e r n e l . A p p l i c a t i o n _ S c h e d u l i n g _ D a t a'
--
--                                   Spec
--
--
--
--  File 'k-appsched_data.ads'                                         By MAR.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------

with Ada.Unchecked_Conversion;
with General_Constants;
with Kernel.Mutexes;
with Kernel.Signals;
with Kernel.Timers;
with Option_Sets;
pragma Elaborate_All (Option_Sets);

package Kernel.Application_Scheduling_Data is

   package GC renames General_Constants;

   ---------------------------------------------------------------------------
   -- Type 'AppSched_Event' --------------------------------------------------
   ---------------------------------------------------------------------------
   type Event_Code_T is
      (APPSCHED_NEW,
       APPSCHED_TERMINATE,
       APPSCHED_READY,
       APPSCHED_BLOCK,
       APPSCHED_YIELD,
       APPSCHED_SIGNAL,
       APPSCHED_CHANGE_SCHED_PARAM,
       APPSCHED_EXPLICIT_CALL,
       APPSCHED_EXPLICIT_CALL_WITH_DATA,
       APPSCHED_TIMEOUT,
       APPSCHED_PRIORITY_INHERIT,
       APPSCHED_PRIORITY_UNINHERIT,
       APPSCHED_INIT_MUTEX,
       APPSCHED_DESTROY_MUTEX,
       APPSCHED_LOCK_MUTEX,
       APPSCHED_TRY_LOCK_MUTEX,
       APPSCHED_UNLOCK_MUTEX,
       APPSCHED_BLOCK_AT_MUTEX,
       APPSCHED_CHANGE_SCHED_PARAM_MUTEX,
       APPSCHED_TASK_NOTIFICATION);
   for Event_Code_T use
      (APPSCHED_NEW                     => GC.APPSCHED_NEW,
       APPSCHED_TERMINATE               => GC.APPSCHED_TERMINATE,
       APPSCHED_READY                   => GC.APPSCHED_READY,
       APPSCHED_BLOCK                   => GC.APPSCHED_BLOCK,
       APPSCHED_YIELD                   => GC.APPSCHED_YIELD,
       APPSCHED_SIGNAL                  => GC.APPSCHED_SIGNAL,
       APPSCHED_CHANGE_SCHED_PARAM      => GC.APPSCHED_CHANGE_SCHED_PARAM,
       APPSCHED_EXPLICIT_CALL           => GC.APPSCHED_EXPLICIT_CALL,
       APPSCHED_EXPLICIT_CALL_WITH_DATA => GC.APPSCHED_EXPLICIT_CALL_WITH_DATA,
       APPSCHED_TIMEOUT                 => GC.APPSCHED_TIMEOUT,
       APPSCHED_PRIORITY_INHERIT        => GC.APPSCHED_PRIORITY_INHERIT,
       APPSCHED_PRIORITY_UNINHERIT      => GC.APPSCHED_PRIORITY_UNINHERIT,
       APPSCHED_INIT_MUTEX              => GC.APPSCHED_INIT_MUTEX,
       APPSCHED_DESTROY_MUTEX           => GC.APPSCHED_DESTROY_MUTEX,
       APPSCHED_LOCK_MUTEX              => GC.APPSCHED_LOCK_MUTEX,
       APPSCHED_TRY_LOCK_MUTEX          => GC.APPSCHED_TRY_LOCK_MUTEX,
       APPSCHED_UNLOCK_MUTEX            => GC.APPSCHED_UNLOCK_MUTEX,
       APPSCHED_BLOCK_AT_MUTEX          => GC.APPSCHED_BLOCK_AT_MUTEX,
       APPSCHED_CHANGE_SCHED_PARAM_MUTEX
                                   => GC.APPSCHED_CHANGE_SCHED_PARAM_MUTEX,
       APPSCHED_TASK_NOTIFICATION       => GC.APPSCHED_TASK_NOTIFICATION);
   for Event_Code_T'Size use Int'Size;

   ---------------------------
   -- type 'AppSched_Event' --
   ---------------------------
   type AppSched_Event is tagged record
      Event_Code           : Event_Code_T;
      T                    : Task_Id;
      Sched_Priority       : Task_Priority;
      M                    : Mutexes.Mutex_Descriptor;
      Siginfo              : Signals.Siginfo_T;
      Info                 : System.Address;
      Info_Size            : Int;
      User_Event_Code      : Int;
   end record;
   package AppSched_Events_Lists is new Singly_Linked_Lists (AppSched_Event);
   subtype AppSched_Event_Ac is AppSched_Events_Lists.Element_Ac;
   --  Changes in the 'AppSched_Event' type must be reflected in the types
   --  'Sched.Posix_Appsched_Event' and its C equivalent
   --  'posix_appsched_event' in 'sched.h'.


   ---------------------------------------------------
   --  Release and request 'AppSched_Event' objects --
   ---------------------------------------------------
   procedure Release_AppSched_Event (E : in AppSched_Event_Ac);
   pragma Inline (Release_AppSched_Event);

   function Request_AppSched_Event return AppSched_Event_Ac;
   pragma Inline (Request_AppSched_Event);


   ----------------------------------------------------------------------------
   -- Type 'Task_Container' ---------------------------------------------------
   ----------------------------------------------------------------------------
   type Task_Container is tagged record
      T : Task_Id;
   end record;
   package Task_Containers_Lists is new Singly_Linked_Lists (Task_Container);
   subtype Task_Container_Ac is Task_Containers_Lists.Element_Ac;

   function ">" (Left, Right : in Task_Container_Ac) return Boolean;
   package Task_Containers_Lists_Order is
     new Task_Containers_Lists.Order (">");

   ---------------------------------------------------
   --  Release and request 'Task_Container' objects --
   ---------------------------------------------------
   procedure Release_Task_Container (TC : in Task_Container_Ac);

   function Request_Task_Container return Task_Container_Ac;

   function Any_Task_Container_Free_In_Pool return Boolean;

   ----------------------------------------------------------------------------
   --  Type 'Scheduler_Reply_T'  ----------------------------------------------
   ----------------------------------------------------------------------------
   type Scheduler_Reply_Size_T is range 0 .. CP.AppSched_Info_Mx;
   for Scheduler_Reply_Size_T'Size use Basic_Integer_Types.Int'Size;
   type Scheduler_Reply_Size_Ac is access all Scheduler_Reply_Size_T;
   type Scheduler_Reply_T is array (Scheduler_Reply_Size_T
                                    range 1 .. Scheduler_Reply_Size_T'Last)
     of Basic_Integer_Types.Unsigned_8;
   pragma Pack (Scheduler_Reply_T);
   pragma Convention (C, Scheduler_Reply_T);
   type Scheduler_Reply_Ac is access all Scheduler_Reply_T;

   ----------------------------------------------------------------------------
   --  Type 'Notification_Timed_Event' ----------------------------------------
   ----------------------------------------------------------------------------
   type Notification_Timed_Event is new Kernel.Timed_Events_Lists.Element
     with record
        Task_To_Notify : Task_Id;
        Used           : Boolean;
     end record;
   Foo_Timed_Events_Lists_Element : Kernel.Timed_Events_Lists.Element;
   Null_Notification_TE : constant Notification_Timed_Event :=
     (Foo_Timed_Events_Lists_Element with
      Task_To_Notify => null,
      Used           => False);

   ----------------------------------------------------------------------------
   --  Type 'Sched_App_Data'  -------------------------------------------------
   ----------------------------------------------------------------------------
   --
   --  Specific data asociated with tasks of SCHED_APP policy in
   --  'TCB.Sched_App'.
   type Sched_App_Data is new Kernel.Sched_App_Data_Base with record
      Scheduler : Task_Id;
      --  Application scheduler task.

      Param : aliased Kernel.AppSched_Param_T;
      --  Application-defined scheduling parameters.

      Param_Size : Kernel.AppSched_Param_Size_T;
      --  Size of the application-defined scheduling parameters.

      AppMutex_Where_Waiting : Mutexes.Mutex_Descriptor;
      --  Used by tasks whose policy is SCHED_APP.

      Scheduler_Reply : Scheduler_Reply_Ac;
      --  Points to the memory area provided for the task to store
      --  the appscheduler reply.

      Reply_Size : Scheduler_Reply_Size_T;
      --  Size in bytes of the appscheduler reply.

      Owner : Task_Id;
      --  Task associated with this record

      Notification_Event : aliased Notification_Timed_Event;
      --  Timed event used to program a task notification event
   end record;
   package Sched_App_Data_Lists is new Singly_Linked_Lists (Sched_App_Data);
   subtype Sched_App_Data_Ac is Sched_App_Data_Lists.Element_Ac;

   function UC_To_Sched_App_Data_Ac is
     new Ada.Unchecked_Conversion (Kernel.Sched_App_Data_Base_Ac,
                                   Sched_App_Data_Ac);

   ---------------------------------------------------
   --  Request 'AppSchedulerSched_App_Data' objects --
   ---------------------------------------------------
   function Request_Sched_App_Data return Sched_App_Data_Ac;

   ----------------------------------------------------------------------------
   -- Type 'AppScheduler_Data' ------------------------------------------------
   ----------------------------------------------------------------------------
   --
   --  Data structure associated with application-scheduler tasks in
   --  'TCB.AppScheduler'.
   type AppScheduler_Data is new Kernel.AppScheduler_Data_Base with record
      Tasks_Q  : Task_Containers_Lists.List := Task_Containers_Lists.Null_List;
      Events_Q : AppSched_Events_Lists.List := AppSched_Events_Lists.Null_List;
      Flags       : Option_Sets.Option_Set := Option_Sets.Empty_Option_Set;
      Events_Mask : Option_Sets.Option_Set := Option_Sets.Empty_Option_Set;
      Waiting_For_Signals : Boolean := False;
      Clock : Timers.Clock_Id := Timers.CLOCK_REALTIME;
      Reserved_TCs : Task_Containers_Lists.List
        := Task_Containers_Lists.Null_List;
      Suspension_Deferred_Tasks : Sched_App_Data_Lists.List
        := Sched_App_Data_Lists.Null_List;
      Reply      : Scheduler_Reply_T      := (others => 0);
      Reply_Size : Scheduler_Reply_Size_T := 0;
   end record;
   package AppScheduler_Data_Lists is
     new Singly_Linked_Lists (AppScheduler_Data);
   subtype AppScheduler_Data_Ac is AppScheduler_Data_Lists.Element_Ac;

   --------------------------------------
   -- 'AppScheduler_Data.Flags' values --
   --------------------------------------
   APPSCHED_ABSTIMEOUT : constant Option_Sets.Option_Set :=
     Option_Sets.To_Options (General_Constants.APPSCHED_ABSTIMEOUT);

   ---------------------
   -- 'To_Option_Set' --
   ---------------------
   --
   --  Transform an 'Event_Code_T' into an 'Option_Set_T' that only contains
   --  the option corresponding to the 'Event_Code_T'
   function To_Option_Set (Event : in Event_Code_T)
                           return Option_Sets.Option_Set;
   pragma Inline (To_Option_Set);

   ------------------------------------------------------
   --  Release and request 'AppScheduler_Data' objects --
   ------------------------------------------------------
   procedure Release_AppScheduler_Data
     (AppScheduler_Data : in AppScheduler_Data_Ac);
   --  It also releases the tasks containers and the events in its
   --  'Tasks_Q' and 'Events_Q' queues.

   function Request_AppScheduler_Data return AppScheduler_Data_Ac;

   ---------------------------------------------
   --  Unchecked Conversion from              --
   --  'Kernel.AppScheduler_Data_Base_Ac' to  --
   --  'AppScheduler_Data_Ac'                 --
   ---------------------------------------------
   function UC_To_AppScheduler_Data_Ac is
     new Ada.Unchecked_Conversion (Kernel.AppScheduler_Data_Base_Ac,
                                   AppScheduler_Data_Ac);


end Kernel.Application_Scheduling_Data;
