------------------------------------------------------------------------------
-- -------------------         M a R T E   O S         -------------------- --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--                    'K e r n e l . P o o l _ T C B s'
--
--                                 Spec
--
--
--  File 'k-pool_tcbs.ads'                                            By MAR.
--
--
--  Free Task Control Blocks management.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------
with Configuration_Parameters;

package Kernel.Pool_TCBs is

   pragma Elaborate_Body;

   package CP renames Configuration_Parameters;

   type TCBs_Range is range 0 .. CP.Num_MaRTE_Tasks_Mx - 1;
   --  'Num_Tasks_Mx' user's task (included the main) plus the idle and
   --  the signal handler tasks.

   --  TCBs preassigned to expecial tasks
   Main_Task_Id : constant := 0;
   Signal_Handler_Task_Id : constant := CP.Num_MaRTE_Tasks_Mx - 2;
   Idle_Task_Id : constant := CP.Num_MaRTE_Tasks_Mx - 1;

   subtype Kernel_Tasks_TCBs_Range is TCBs_Range
     range Configuration_Parameters.Num_User_Tasks_Mx .. TCBs_Range'Last;
   --  Kernel tasks IDs (Idle and signal handler tasks)
   subtype User_Tasks_TCBs_Range is TCBs_Range
      range 0 .. Configuration_Parameters.Num_User_Tasks_Mx - 1;
   --  User tasks IDs (including the main)
   subtype Normal_Tasks_TCBs_Range is TCBs_Range
      range 1 .. Configuration_Parameters.Num_User_Tasks_Mx - 1;
   --  Normal tasks IDs (all but the main, idle and signal handler)

   procedure Release_TCB (T : in Task_Id);
   pragma Inline (Release_TCB);

   function Request_TCB return Task_Id;
   pragma Inline (Request_TCB);

   function Get_TCB_By_Number (Num : TCBs_Range) return Task_Id;
   --  Can be used to get one by one every TCB and perform some action on it.

end Kernel.Pool_TCBs;
