------------------------------------------------------------------------------
----------------------         M a R T E   O S         -----------------------
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--             'K e r n e l . S i g n a l s . P o o l _ S D B s'
--
--                                 Body
--
--
--  File 'k-signals-pool_sdbs.adb'                                     By Mar.
--
--
--  Free Signal Data Blocks (SDB) management.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
-------------------------------------------------------------------------------
with Configuration_Parameters;
with Singly_Linked_Lists.Resources;
pragma Elaborate_All (Singly_Linked_Lists.Resources);

with Kernel.Signals.Global;
use Kernel.Signals.Global;

package body Kernel.Signals.Pool_SDBs is


   --------------------------------------------------------------------------
   -- Pool of Signal Data Blocks (SDB) --------------------------------------
   --------------------------------------------------------------------------
   use Kernel.Signals.Global.Sig_Data_Lists;
   Free_SDBs : array (1 .. Configuration_Parameters.Num_User_Tasks_Mx)
     of aliased Sig_Data_Lists.Element;
   package SDBs_Mangement is new Signals.Global.Sig_Data_Lists.Resources;


   Sig_Data_Default_Value : aliased constant Sig_Data :=
     (Mask                => Blockeable_Signals,
      Pending_Signals  => (others => Global.Signal_Instances_Lists.Null_List),
      Waited_Signals => (others => False),
      Is_Waiting_For_Signals => False,
      Accepted_Siginst_Ac => null,
      Owner               => null);

   -----------------
   -- Request_SDB --
   -----------------
   function Request_SDB return Kernel.Signals.Global.Sig_Data_Ac
     renames SDBs_Mangement.Request;

   -------------------------
   --  Register_Used_SDB  --
   -------------------------
   procedure Register_Used_SDB (T : in Kernel.Task_Id) is
      use type Kernel.Task_Id;
   begin
      pragma Assert (T.Sig_Data /= null        and then
                       Sig_Data_Ac (T.Sig_Data).Owner = T);
      pragma Assert (not Sig_Data_Lists.Is_In_The_List
                       (Sig_Data_Ac (T.Sig_Data),
                        Sig_Data_Blocks_Used));
      Sig_Data_Lists.Enqueue_Head (Sig_Data_Ac (T.Sig_Data),
                                   Sig_Data_Blocks_Used);
   end Register_Used_SDB;

begin
   --  Initialize 'Free_SDBs'
   for I in Free_SDBs'Range loop
      SDBs_Mangement.Release (Sig_Data_Default_Value'Access,
                              Free_SDBs (I)'Access);
   end loop;
end Kernel.Signals.Pool_SDBs;
