------------------------------------------------------------------------------
-- -------------------         M a R T E   O S         -------------------- --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--                  'K e r n e l . T y p e s _ S i z e s'
--
--                                 Spec
--
--
--  File 'k-types_sizes.ads'                                           By MAR.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
-------------------------------------------------------------------------------
with Kernel.Tasks_Operations;
with Kernel.Mutexes;
with Kernel.Condition_Variables;
with Kernel.Semaphores;
with Kernel.Signals;
with Kernel.Tasks_Operations.Application_Scheduler;
with Time;
with Sched;
with Pthread;
with Pthread_Once;
use Kernel;
with Option_Sets;

package Kernel.Types_Sizes is

   --  Kernel
   Kernel_TCB_Size_In_Bytes : constant Integer :=
     Kernel.TCB'Size / 8;
   pragma Export (C, Kernel_TCB_Size_In_Bytes,
                  "KERNEL_TCB_SIZE_IN_BYTES");

   --  Tasks_Operations
   Sched_Param_Size_In_Bytes : constant Integer :=
     Tasks_Operations.Sched_Param'Size / 8;
   pragma Export (C, Sched_Param_Size_In_Bytes, "SCHED_PARAM_SIZE_IN_BYTES");

   Attr_Size_In_Bytes : constant Integer :=
     Tasks_Operations.Pthread_Attr_T'Size / 8;
   pragma Export (C, Attr_Size_In_Bytes, "PTHREADATTR_SIZE_IN_BYTES");

   --  Mutexes
   Mutex_Size_In_Bytes : constant Integer := Mutexes.Mutex'Size / 8;
   pragma Export (C, Mutex_Size_In_Bytes, "MUTEX_SIZE_IN_BYTES");

   Mutex_Attr_Size_In_Bytes : constant Integer := Mutexes.Attributes'Size / 8;
   pragma Export (C, Mutex_Attr_Size_In_Bytes, "MUTEXATTR_SIZE_IN_BYTES");

   Mutex_AppSched_Param_T_Size_In_Bytes : constant Integer :=
     Mutexes.Mutex_AppSched_Param_T'Size / 8;
   pragma Export (C, Mutex_AppSched_Param_T_Size_In_Bytes,
                  "MUTEX_APPSCHED_PARAM_T_SIZE_IN_BYTES");

   --  Condition_Variables
   CV_Size_In_Bytes : constant Integer :=
     Condition_Variables.Condition'Size / 8;
   pragma Export (C, CV_Size_In_Bytes, "CV_SIZE_IN_BYTES");

   CV_Attr_Size_In_Bytes : constant Integer :=
     Condition_Variables.Attributes'Size / 8;
   pragma Export (C, CV_Attr_Size_In_Bytes, "CVATTR_SIZE_IN_BYTES");

   --  Semaphores
   Sem_Size_In_Bytes : constant Integer :=
     Kernel.Semaphores.Semaphore'Size / 8;
   pragma Export (C, Sem_Size_In_Bytes, "SEM_SIZE_IN_BYTES");

   --  Signals
   Signal_Set_Size_In_Bytes : constant Integer := Signals.Signal_Set'Size / 8;
   pragma Export (C, Signal_Set_Size_In_Bytes, "SIGNAL_SET_SIZE_IN_BYTES");

   Struct_Sig_Action_Size_In_Bytes : constant Integer :=
     Signals.Struct_Sig_Action'Size / 8;
   pragma Export (C, Struct_Sig_Action_Size_In_Bytes,
                  "STRUCT_SIG_ACTION_SIZE_IN_BYTES");

   Siginfo_T_Size_In_Bytes : constant Integer :=
     Signals.Siginfo_T'Size / 8;
   pragma Export (C, Siginfo_T_Size_In_Bytes, "SIGINFO_T_SIZE_IN_BYTES");

   Signal_Event_Size_In_Bytes : constant Integer :=
     Signals.Signal_Event'Size / 8;
   pragma Export (C, Signal_Event_Size_In_Bytes,
                  "SIGNAL_EVENT_SIZE_IN_BYTES");

   --  Time
   Itimerspec_Size_In_Bytes : constant Integer :=
     Time.Itimerspec'Size / 8;
   pragma Export (C, Itimerspec_Size_In_Bytes,
                  "ITIMERSPEC_SIZE_IN_BYTES");

   --  Tasks_Operations.Application_Scheduler
   Sched_Actions_Set_Size_In_Bytes : constant Integer :=
     Tasks_Operations.Application_Scheduler.Sched_Actions_Set'Size / 8;
   pragma Export (C, Sched_Actions_Set_Size_In_Bytes,
                  "SCHED_ACTIONS_SET_SIZE_IN_BYTES");

   --  Sched
   Posix_Appsched_Event_Size_In_Bytes : constant Integer :=
     Sched.Posix_Appsched_Event'Size / 8;
   pragma Export (C, Posix_Appsched_Event_Size_In_Bytes,
                  "POSIX_APPSCHED_EVENT_SIZE_IN_BYTES");
   Posix_Appsched_Eventset_T_Size_In_Bytes : constant Integer :=
     Option_Sets.Option_Set'Size / 8;
   pragma Export (C, Posix_Appsched_Eventset_T_Size_In_Bytes,
                  "POSIX_APPSCHED_EVENTSET_T_SIZE_IN_BYTES");

   --  Pthread
   Pthread_Key_T_Size_In_Bytes : constant Integer :=
     Pthread.Pthread_Key_T'Size / 8;
   pragma Export (C, Pthread_Key_T_Size_In_Bytes,
                    "PTHREAD_KEY_T_SIZE_IN_BYTES");

   Pthread_Once_Size_In_Bytes : constant Integer :=
     Pthread_Once.Pthread_Once_T'Size / 8;
   pragma Export (C, Pthread_Once_Size_In_Bytes,
                    "PTHREAD_ONCE_SIZE_IN_BYTES");

end Kernel.Types_Sizes;
