------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--                   'S t a c k s _ M a n a g e m e n t'
--
--                                 Spec
--
--
--  File 'stacks_management.ads'                                       By Mar.
--
--
--  Task stacks management.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------
with System; --  for 'Address'
with Basic_Integer_Types; use Basic_Integer_Types;
with Configuration_Parameters;

package Stacks_Management is

   type Stack_ID is private;

   Null_Stack : constant Stack_ID;

   --  XXX All stacks have the same size.
   Stack_Size_In_Bytes : constant Integer :=
     Configuration_Parameters.Stack_Size_In_Bytes;
   Stack_Size_In_DWords : constant Integer := Stack_Size_In_Bytes / 4;

   type DWord_Pos is range 1 .. Stack_Size_In_DWords;

   function Request_Stack (Size : in Integer) return Stack_ID;

   procedure Release_Stack (The_Stack : in Stack_ID);

   function Stack_Base_Address (The_Stack : in Stack_ID)
                                return Unsigned_32;

   function Stack_Top_Address (The_Stack : in Stack_ID)
                               return Unsigned_32;

   function DWord_In_Stack_Address (The_Pos      : in DWord_Pos;
                                    In_The_Stack : in Stack_ID)
                                    return Unsigned_32;

   function Get_Stack_Size_In_Bytes (The_Stack : in Stack_ID) return Integer;

   procedure Write_In_Stack (Addr       : in System.Address;
                             In_The_Pos : in DWord_Pos;
                             In_Stack   : in Stack_ID);

   procedure Write_In_Stack (Int        : in Integer;
                             In_The_Pos : in DWord_Pos;
                             In_Stack   : in Stack_ID);

   function Read_From_Stack (Pos   : in DWord_Pos;
                             Stack : in Stack_ID) return Integer;

   function Stack_Overflow (Stack : in Stack_ID) return Boolean;


private

   type Stack_Data;

   type Stack_ID is access all Stack_Data;

   Null_Stack : constant Stack_ID := null;

end Stacks_Management;
