------------------------------------------------------------------------------
-- --------------------        M a R T E   O S          ------------------- --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--                         'S i g n a l s _ I n f o'
--
--                                   Spec
--
--
--  File 'signals_info.ads'                                            By MAR.
--
--  Description strings for the MaRTE signals.
--
--  The list defined here is used by 'write_marte_c_headers.adb' to
--  write automatically during the MaRTE installation the file
--  'sys/marte_signals.h'.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------

with Signals_Values;

package Signals_Info is

   type Signals_Names is (SIGNULL,
                          SIGHUP,
                          SIGINT,
                          SIGQUIT,
                          SIGILL,
                          SIGTRAP,
                          SIGABRT,
                          SIGBUS,
                          SIGFPE,
                          SIGKILL,
                          SIGUSR1,
                          SIGSEGV,
                          SIGUSR2,
                          SIGPIPE,
                          SIGALRM,
                          SIGTERM,
                          SIGCHLD,
                          SIGCONT,
                          SIGSTOP,
                          SIGTSTP,
                          SIGTTIN,
                          SIGTTOU,
                          SIGVTALRM,
                          SIGPROF,
                          SIGUNUSED,
                          SIGRTMIN,
                          SIGRTMAX);

   type Signal_Info is record
      Number      : Integer;
      Description : String (1..45);
   end record;

   List : constant array (Signals_Names) of Signal_Info :=
     (SIGNULL   =>
        (Number => Signals_Values.SIGNULL,
         Description => "null signal                                  "),
      SIGHUP    =>
        (Number => Signals_Values.SIGHUP,
         Description => "hangup                                       "),
      SIGINT    =>
        (Number => Signals_Values.SIGINT,
         Description => "interrupt                                    "),
      SIGQUIT   =>
        (Number => Signals_Values.SIGQUIT,
         Description => "quit                                         "),
      SIGILL    =>
        (Number => Signals_Values.SIGILL,
         Description => "illegal instruction                          "),
      SIGTRAP   =>
        (Number => Signals_Values.SIGTRAP,
         Description => "trace trap                                   "),
      SIGABRT   =>
        (Number => Signals_Values.SIGABRT,
         Description => "abort()                                      "),
      SIGBUS    =>
        (Number => Signals_Values.SIGBUS,
         Description => "bus error                                    "),
      SIGFPE    =>
        (Number => Signals_Values.SIGFPE,
         Description => "floating point exception                     "),
      SIGKILL   =>
        (Number => Signals_Values.SIGKILL,
         Description => "kill (cannot be caught or ignored)           "),
      SIGUSR1   =>
        (Number => Signals_Values.SIGUSR1,
         Description => "user defined signal 1                        "),
      SIGSEGV   =>
        (Number => Signals_Values.SIGSEGV,
         Description => "segmentation violation                       "),
      SIGUSR2   =>
        (Number => Signals_Values.SIGUSR2,
         Description => "user defined signal 2                        "),
      SIGPIPE   =>
        (Number => Signals_Values.SIGPIPE,
         Description => "write on a pipe with no one to read it       "),
      SIGALRM   =>
        (Number => Signals_Values.SIGALRM,
         Description => "alarm clock                                  "),
      SIGTERM   =>
        (Number => Signals_Values.SIGTERM,
         Description => "Termination Signal                           "),
      SIGCHLD   =>
        (Number => Signals_Values.SIGCHLD,
         Description => "to parent on child stop or exit              "),
      SIGCONT   =>
        (Number => Signals_Values.SIGCONT,
         Description => "continue a stopped process                   "),
      SIGSTOP   =>
        (Number => Signals_Values.SIGSTOP,
         Description => "sendable stop signal not from tty            "),
      SIGTSTP   =>
        (Number => Signals_Values.SIGTSTP,
         Description => "stop signal from tty                         "),
      SIGTTIN   =>
        (Number => Signals_Values.SIGTTIN,
         Description => "Read from ctr term by member of bg proc group"),
      SIGTTOU   =>
        (Number => Signals_Values.SIGTTOU,
         Description => "Write to ctr term by member of bg proc group "),
      SIGVTALRM =>
        (Number => Signals_Values.SIGVTALRM,
         Description => "virtual timer expired                        "),
      SIGPROF =>
        (Number => Signals_Values.SIGPROF,
         Description => "PROF timer expired                           "),
      SIGUNUSED =>
        (Number => Signals_Values.SIGUNUSED,
         Description => "unused signal                                "),
      SIGRTMIN =>
        (Number => Signals_Values.SIGRTMIN,
         Description => "minimum value for the RT signals             "),
      SIGRTMAX =>
        (Number => Signals_Values.SIGRTMAX,
         Description => "maximum value for the RT signals             "));

end Signals_Info;
