------------------------------------------------------------------------------
--                                                                          --
--                GNU ADA RUN-TIME LIBRARY (GNARL) COMPONENTS               --
--                                                                          --
--                   S Y S T E M . O S _ I N T E R F A C E                  --
--                                                                          --
--                                  B o d y                                 --
--                                                                          --
--                              $Revision: 1.5 $
--                                                                          --
--            Copyright (C) 1997-2000 Florida State University              --
--                                                                          --
-- GNARL is free software; you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. GNARL is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNARL; see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- GNARL was developed by the GNARL team at Florida State University. It is --
-- now maintained by Ada Core Technologies Inc. in cooperation with Florida --
-- State University (http://www.gnat.com).                                  --
--                                                                          --
------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--  This is a MaRTE OS version of this package.
--
--  (Thanks to Javier Miranda (Universidad de Las Palmas de Gran Canaria)
--  for its help in this adaptation).
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------

pragma Polling (Off);
--  Turn off polling, we do not want ATC polling to take place during
--  tasking operations. It causes infinite loops and other problems.

with Ada.Unchecked_Conversion;


with Stacks_Management;
pragma Elaborate_All (Stacks_Management);

with Hardware_Interface;
with Sched;
pragma Elaborate_All (Sched);
with Time;
pragma Elaborate_All (Time);
with Pthread;
pragma Elaborate_All (Pthread);

with Kernel_Console;

with Interfaces.C;

package body System.OS_Interface is

   use Interfaces.C;
   package HWI renames Hardware_Interface;

   ----------------------
   -- Type conversions --
   ----------------------
   function To_Descriptor (M : access pthread_mutex_t) return
     Pthread.Mutex_Descriptor;

   function To_Descriptor (M : access pthread_mutex_t) return
     Pthread.Mutex_Descriptor is
   begin
      return Pthread.Descriptor_Of (Pthread.Pthread_Mutex_T (M.all));
   end To_Descriptor;
   pragma Inline (To_Descriptor);

   function To_Descriptor (CV : access pthread_cond_t) return
     Pthread.Condition_Descriptor;

   function To_Descriptor (CV : access pthread_cond_t) return
     Pthread.Condition_Descriptor is
   begin
      return Pthread.Descriptor_Of
        (Pthread.Pthread_Cond_T (CV.all));
   end To_Descriptor;
   pragma Inline (To_Descriptor);

   type Mutex_Ac is access Pthread.Pthread_Mutex_T;
   function To_Access_Mutex (M : access pthread_mutex_t) return Mutex_Ac;
   pragma Import (C, To_Access_Mutex, "convert_4_bytes_long_data");

   function To_Gnat_Pthread_T is
     new Ada.Unchecked_Conversion (Pthread.Pthread_T, pthread_t);
   function To_MaRTE_Pthread_T is
     new Ada.Unchecked_Conversion (pthread_t, Pthread.Pthread_T);

   -----------
   -- Errno --
   -----------
   function errno return int is
   begin
      return Pthread.Pthread_Self.Error_Code;
   end errno;

   ----------------------------------------------------------------------------
   -- Signals -----------------------------------------------------------------
   ----------------------------------------------------------------------------

   function sigaddset
     (set : access sigset_t;
      sig : in Signal)
      return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret := PSignal.Sigaddset (Set_Tmp'Access, PSignal.Signal (sig));
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigaddset;

   function sigdelset
     (set : access sigset_t;
      sig : in Signal)
      return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret := PSignal.Sigdelset (Set_Tmp'Access, PSignal.Signal (sig));
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigdelset;

   function sigfillset
     (set : access sigset_t)
      return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret :=  PSignal.Sigfillset (Set_Tmp'Access);
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigfillset;

   function sigismember
     (set : access sigset_t;
      sig : Signal)
     return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret := PSignal.Sigismember (Set_Tmp'Unchecked_Access,
                                  PSignal.Signal (sig));
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigismember;

   function sigemptyset
     (set : access sigset_t)
     return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret :=  PSignal.Sigemptyset (Set_Tmp'Access);
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigemptyset;

   ---------------
   -- sigaction --
   ---------------
   function To_Struct_Sig_Action_Ac is
     new Ada.Unchecked_Conversion (struct_sigaction_ptr,
                                   PSignal.Struct_Sig_Action_Ac);
   function To_Struct_Sig_Action_Ac_Const is
     new Ada.Unchecked_Conversion (struct_sigaction_ptr,
                                   PSignal.Struct_Sig_Action_Ac_Const);

   function sigaction (sig  : Signal;
                       act  : struct_sigaction_ptr;
                       oact : struct_sigaction_ptr) return int is
   begin
      return PSignal.Sigaction (PSignal.Signal (sig),
                                To_Struct_Sig_Action_Ac_Const (act),
                                To_Struct_Sig_Action_Ac (oact));
   end sigaction;


   ----------------------------------------------------------------------------
   -- Time --------------------------------------------------------------------
   ----------------------------------------------------------------------------

   function clock_gettime (clock_id : clockid_t;
                           tp       : access timespec) return int is
      use type Time.Clockid_T;
   begin
      pragma Assert (clock_id = CLOCK_REALTIME);
      tp.all := Time.Get_Time_Gnat;
      return 0;
   end clock_gettime;

   function To_Duration (TS : timespec) return Duration is
   begin
      return TS;
   end To_Duration;
   pragma Inline (To_Duration);

   function To_Timespec (D : Duration) return timespec is
   begin
      return D;
   end To_Timespec;
   pragma Inline (To_Timespec);


   ---------------------------------------------------------------------------
   -- Process ----------------------------------------------------------------
   ---------------------------------------------------------------------------

   --  XXX Not Implemented
   function getpid return pid_t is
   begin
      return 0;
   end getpid;

   --  << MaRTE OS
   ----------------------------------------------------------------------------
   -- LWP (MARTE) -------------------------------------------------------------
   ----------------------------------------------------------------------------

   function To_System_Address is
      new Ada.Unchecked_Conversion (pthread_t, System.Address);

   function lwp_self return System.Address is
   --  lwp_self does not exist on this thread library, revert to pthread_self
   --  which is the closest approximation (with getpid).
      Self_ID : pthread_t := pthread_self;
   begin
      return To_System_Address (Self_ID);
   end lwp_self;
   --  MaRTE OS >>


   ----------------------------------------------------------------------------
   -- Stack -------------------------------------------------------------------
   ----------------------------------------------------------------------------

   function Get_Stack_Base (thread : pthread_t) return Address is
   begin
      return System.Null_Address;
      --  returns the stack base of the specified thread.
      --  Only call this function when Stack_Base_Available is True.
   end Get_Stack_Base;
   pragma Inline (Get_Stack_Base);

   function Get_Page_Size return size_t is
   begin
      return 0;
      --  returns the size of a page, or 0 if this is not relevant on this
      --  target
   end Get_Page_Size;

   function Get_Page_Size return Address is
   begin
      return System.Null_Address;
      --  returns the size of a page, or 0 if this is not relevant on this
      --  target
   end Get_Page_Size;

   --  XXX Not Implemented
   function mprotect (addr : Address; len : size_t; prot : int) return int is
   begin
      Kernel_Console.Error ("Called Not Implemented function: Mprotect !!");
      return 0;
   end mprotect;

   ---------------------------------------------------------------------------
   --  Nonstandard Thread Initialization  ------------------------------------
   ---------------------------------------------------------------------------
   --  FSU_THREADS requires pthread_init, which is nonstandard
   --  and this should be invoked during the elaboration of s-taprop.adb
   procedure pthread_init is
   begin
      null;
   end pthread_init;

   ---------------------------------------------------------------------------
   --  POSIX.1c  Section 3  --------------------------------------------------
   ---------------------------------------------------------------------------

   ---------------
   --  sigwait  --
   ---------------
   function sigwait
     (set : access sigset_t;
      sig : access Signal)
      return int is
      Sig_Tmp : aliased PSignal.Signal;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
      Ret : int;
   begin
      Ret := PSignal.Sigwait (Set_Tmp'Unchecked_Access, Sig_Tmp'Access);
      sig.all := Signal (Sig_Tmp);
      return Ret;
   end sigwait;

   --------------------
   --  pthread_kill  --
   --------------------
   function pthread_kill
     (thread : pthread_t;
      sig    : Signal)
      return   int is
   begin
      return PSignal.Pthread_Kill (To_MaRTE_Pthread_T (thread),
                                   PSignal.Signal (sig));
   end pthread_kill;

   -----------------------
   --  pthread_sigmask  --
   -----------------------
   function To_Signal_Set_Ac is
     new Ada.Unchecked_Conversion (sigset_t_ptr, PSignal.Signal_Set_Ac);
   function To_Signal_Set_Ac_Const is
     new Ada.Unchecked_Conversion (sigset_t_ptr, PSignal.Signal_Set_Ac_Const);

   function pthread_sigmask
     (how  : int;
      set  : sigset_t_ptr;
      oset : sigset_t_ptr)
      return int is
   begin
      return PSignal.Pthread_Sigmask (Integer (how),
                                      To_Signal_Set_Ac_Const (set),
                                      To_Signal_Set_Ac (oset));
   end pthread_sigmask;


   ---------------------------------------------------------------------------
   --  POSIX.1c  Section 11  -------------------------------------------------
   ---------------------------------------------------------------------------

   function pthread_mutex_init (mutex : access pthread_mutex_t;
                                attr  : access pthread_mutexattr_t)
                                return int is
   begin
      return Pthread.Pthread_Mutex_Init (To_Access_Mutex (mutex), attr);
   end pthread_mutex_init;

   function pthread_mutex_destroy (mutex : access pthread_mutex_t)
                                   return  int is
   begin
      return Pthread.Pthread_Mutex_Destroy (To_Access_Mutex (mutex));
   end pthread_mutex_destroy;

   function pthread_mutex_lock (mutex : access pthread_mutex_t)
                                return int is
   begin
      return Pthread.Pthread_Mutex_Lock (To_Descriptor (mutex));
   end pthread_mutex_lock;

   function pthread_mutex_unlock (mutex : access pthread_mutex_t)
                                  return int is
   begin
      return Pthread.Pthread_Mutex_Unlock (To_Descriptor (mutex));
   end pthread_mutex_unlock;

   function pthread_cond_wait (cond  : access pthread_cond_t;
                               mutex : access pthread_mutex_t)
                               return  int is
   begin
      return Pthread.Pthread_Cond_Wait (To_Descriptor (cond),
                                        To_Descriptor (mutex));
   end pthread_cond_wait;

   function pthread_cond_timedwait (cond    : access pthread_cond_t;
                                    mutex   : access pthread_mutex_t;
                                    abstime : access timespec)
                                    return    int is
   begin
      return Pthread.Pthread_Cond_Timedwait_Gnat
        (To_Descriptor (cond),
         To_Descriptor (mutex),
         HWI.Duration_To_HWTime (abstime.all));
   end pthread_cond_timedwait;


   ---------------------------------------------------------------------------
   --  POSIX.1c  Section 13  -------------------------------------------------
   ---------------------------------------------------------------------------

   function pthread_getschedparam (thread : pthread_t;
                                   policy : access int;
                                   param  : access struct_sched_param)
                                   return int is
      Policy_Tmp : aliased Pthread.Scheduling_Policies;
      Param_Tmp : aliased Pthread.Sched_Param;
      Ret : int;
   begin
      Ret := Pthread.Pthread_Getschedparam (To_MaRTE_Pthread_T (thread),
                                            Policy_Tmp'Access,
                                            Param_Tmp'Access);
      policy.all := int (Policy_Tmp);
      param.sched_priority := int (Param_Tmp.Sched_Priority);
      return Ret;
   end pthread_getschedparam;

   function pthread_setschedparam (thread : pthread_t;
                                   policy : int;
                                   param  : access struct_sched_param)
                                   return int is
   begin
      return Pthread.Pthread_Setschedparam_Gnat
                     (To_MaRTE_Pthread_T (thread),
                      Pthread.Scheduling_Policies (policy),
                      Pthread.Task_Priority (param.sched_priority));
      --  Gnat only knows about the 'sched_priority' param, so the normal
      --  'Pthread.Pthread_Setschedparam' can not be called from here.
   end pthread_setschedparam;

   ----------------------------------------------------------------------------
   --  P1003.1c - Section 16  -------------------------------------------------
   ----------------------------------------------------------------------------

   --  XXX Not Implemented
   function pthread_attr_getstacksize
     (attr     : access pthread_attr_t;
      stacksize : access size_t)
      return int is
   begin
      stacksize.all := size_t (Stacks_Management.Stack_Size_In_Bytes);
      return 0;
   end pthread_attr_getstacksize;

   --  XXX Not Implemented
   function pthread_attr_setstacksize
     (attr     : access pthread_attr_t;
      stacksize : size_t)
      return int is
   begin
      return 0;
   end pthread_attr_setstacksize;

   --------------------
   -- Pthread_Create --
   --------------------
   function To_Task_Body_Function is
     new Ada.Unchecked_Conversion (Thread_Body, Pthread.Task_Body_Function);

   function pthread_create (thread        : access pthread_t;
                            attributes    : access pthread_attr_t;
                            start_routine : Thread_Body;
                            arg           : System.Address)
                            return int is
      Thread_Tmp : aliased Pthread.Pthread_T;
      Ret : int;
   begin
      Ret :=
        Pthread.Pthread_Create (Thread_Tmp'Access,
                                Pthread.Pthread_Attr_T_Ac_Const (attributes),
                                To_Task_Body_Function (start_routine),
                                arg);
      thread.all := To_Gnat_Pthread_T (Thread_Tmp);
      return Ret;
   end pthread_create;

   function pthread_detach (thread : pthread_t) return int is
   begin
      return Pthread.Pthread_Detach (To_MaRTE_Pthread_T (thread));
   end pthread_detach;

   function pthread_self return pthread_t is
   begin
      return To_Gnat_Pthread_T (Pthread.Pthread_Self);
   end pthread_self;

   function pthread_equal (t1 : pthread_t; t2 : pthread_t) return int is
   begin
      return Pthread.Pthread_Equal (To_MaRTE_Pthread_T (t1),
                                    To_MaRTE_Pthread_T (t2));
   end pthread_equal;


   ----------------------------------------------------------------------------
   --  POSIX.1c  Section 17  --------------------------------------------------
   ----------------------------------------------------------------------------

   --  Gnat uses the thread specific data functionality to store the gnat
   --  task id (see 'System.Task_Primitives.Operations.Specific.Self').

   Keys_Used_By_Gnat : Integer := 0;
   function Destructor_To_Address is
     new Ada.Unchecked_Conversion (destructor_pointer, System.Address);
   function pthread_key_create
     (key        : access pthread_key_t;
      destructor : destructor_pointer)
      return int is
      Ret  : int;
   begin
      pragma Assert (Keys_Used_By_Gnat = 0,
                     "s-osinte.adb: gnat used more than one key");
      Keys_Used_By_Gnat := Keys_Used_By_Gnat + 1;
      Ret :=
        Pthread.Pthread_Key_Create (key, Destructor_To_Address (destructor));
      Self_Key := key.all;
      --  'Self_Key' is used from
      --  'System.Task_Primitives.Operations.Abort_Handler'.
      return Ret;
   end pthread_key_create;

end System.OS_Interface;
