------------------------------------------------------------------------------
-- -------------------         M a R T E   O S         -------------------- --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--                     'M a R T E _ S e m a p h o r e s'
--
--                                 Spec
--
--
--  File 'marte_semaphores.ads'                                        By MAR.
--
--
--  Direct mapping of semaphores internal functions defined inside
--  MaRTE OS kernel.
--
--  It is mainly intended for Ada drivers that need to use
--  semaphores. Using this interface (instead 'POSIX_Semaphores')
--  leads to a smaller MaRTE footprint since dependency with the
--  POSIX-Ada binding is not included.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
-------------------------------------------------------------------------------
with Basic_Integer_Types; use Basic_Integer_Types;
with Kernel.Semaphores;
with Kernel.Semaphores.Operations;
with Semaphores;
with Type_Timespec;

package MaRTE_Semaphores is

   subtype Semaphore_Value is Kernel.Semaphores.Semaphore_Value;
   subtype Semaphore       is Kernel.Semaphores.Semaphore;
   subtype Semaphore_Ac    is Kernel.Semaphores.Semaphore_ac;
   --  type Semaphore_Ac is access all Semaphore;

   ----------------
   -- Initialize --
   ----------------
   function Initialize (Sem     : in Semaphore_Ac;
			Pshared : in Int;
			Value   : in Semaphore_Value) return Int
     renames Kernel.Semaphores.Operations.Sem_Init;

   -----------------
   -- Sem_Destroy --
   -----------------
   function Destroy (Sem : in Semaphore_Ac) return Int
     renames Kernel.Semaphores.Operations.Sem_Destroy;

   ----------
   -- Wait --
   ----------
   function Wait (Sem : in Semaphore_Ac) return Int
     renames Kernel.Semaphores.Operations.Sem_Wait;

   -------------
   -- Trywait --
   -------------
   function Trywait (Sem : in Semaphore_Ac) return Int
     renames Kernel.Semaphores.Operations.Sem_Trywait;

   ---------------
   -- Timedwait --
   ---------------
   function Timedwait (Sem         : in Semaphore_Ac;
		       Abs_Timeout : in Type_Timespec.Timespec_Ac_Const)
		      return Int
     renames Semaphores.Sem_Timedwait;

   ----------
   -- Post --
   ----------
   function Post (Sem : in Semaphore_Ac) return Int
     renames Kernel.Semaphores.Operations.Sem_Post;

   --------------
   -- Getvalue --
   --------------
   function Getvalue (Sem   : in     Semaphore_Ac;
		      Value : access Semaphore_Value) return Int
     renames Kernel.Semaphores.Operations.Sem_Getvalue;

end MaRTE_Semaphores;
