/*----------------------------------------------------------------------------
 *-------------------------      M a R T E   O S      ------------------------
 *----------------------------------------------------------------------------
 *                                                             V1.57  Dic 2005
 *                                  'f r e e'
 *
 *                                      C
 *
 * File 'free.c'                                     By Miguel Masmano Tello
 *                                                      and MAR.
 *
 * Implements two versions of free:
 *
 * - Two Levels Segregate Fit memory allocator (TLSF): This algorithm
 *   allows memory allocation and deallocation with bounded response
 *   times (Developed at the Universidad Politcnica de Valencia by
 *   Miguel Masmano Tello <mmasmano@disca.upv.es>)
 *
 * - Simple algorithm: allows allocation of memory blocks with not
 *   deallocation. It is useful for applications that does not use
 *   dynamic memory at all or uses it only at initialization time.
 *
 * ---------------------------------------------------------------------------
 *
 * Two Levels Segregate Fit memory allocator (TLSF)
 * Version 1.1
 *
 * Written by Miguel Masmano Tello <mmasmano@disca.upv.es>
 * Copyright (C) Dec, 2002 OCERA Consortium
 * Release under the terms of the GNU General Public License Version 2
 *
 * ----------------------------------------------------------------------
 *  Copyright (C) 2004   Universidad de Cantabria, SPAIN
 *
 *  MaRTE OS web page: http://marte.unican.es
 *  Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
 *                     Michael González Harbour      mgh@unican.es
 *
 * MaRTE OS  is free software; you can  redistribute it and/or  modify it
 * under the terms of the GNU General Public License  as published by the
 * Free Software Foundation;  either  version 2, or (at  your option) any
 * later version.
 *
 * MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
 * WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 * MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 * General Public License for more details.
 *
 * You should have received  a  copy of  the  GNU General Public  License
 * distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
 * Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 * 02111-1307, USA.
 *
 * As a  special exception, if you  link this  unit  with other  files to
 * produce an   executable,   this unit  does  not  by  itself cause  the
 * resulting executable to be covered by the  GNU General Public License.
 * This exception does  not however invalidate  any other reasons why the
 * executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
#include <sys/marte_configuration_parameters.h>

#ifdef USE_TLSF_MEMORY_ALLOCATOR
/*
 * Two Levels Segregate Fit memory allocator (TLSF)
 * Version 1.1
 *
 * Written by Miguel Masmano Tello <mmasmano@disca.upv.es>
 * Copyright (C) Dec, 2002 OCERA Consortium
 * Release under the terms of the GNU General Public License Version 2
 *
 */
#include "tlsf.h"
#include "tlsf_mapping.h"

/*
 * see man free
 *
 * free () is only guaranteed  to work if ptr is the address
 * of a block allocated by rt_malloc() (and not yet freed).
 */
void free (void *ptr) 
{
  int fl, sl, flags;
  block_header_t *bh = NULL, *bh2 = NULL, *bh3;

  // << MaRTE
  //TLSF_t *ptr_TLSF;

  //ptr_TLSF = (TLSF_t *) block_ptr;
  // MaRTE >>

  if (ptr_TLSF == NULL || ptr_TLSF -> magic_number != MAGIC_NUMBER) {
    PRINT_DBG_C ("FATAL ERROR: TLSF structure not initialized\n");
    return;
  }

  bh = (block_header_t *) ((__u8 *) ptr - 
			   TLSF_WORDS2BYTES(beg_header_overhead));
 
  flags = THREAD_LOCK;

  /* now bh is a free block */
  
  SET_FREE_BLOCK (bh);
  bh -> ptr.free_ptr.prev = NULL;
  bh -> ptr.free_ptr.next = NULL;

  /*
   * first of all, we will try to merge bh with the
   * physically contiguos free block and
   * after we will inserte bh into TLSF structure
   */

  /* Now we will try if we can merge bh with the next phys. contiguos block */
  if (!IS_LAST_BLOCK (bh)) {
    /* is it the next block free? */
    /* The next block is easy to found */
    
    bh2 = (block_header_t *) (__u8 *) (bh -> ptr.buffer + 
				       TLSF_WORDS2BYTES
				       (GET_BLOCK_SIZE(bh)));
    if (!IS_USED_BLOCK (bh2)) {
      /* we are lucky, we can merge bh with the following one */  
      if (bh2 -> ptr.free_ptr.next != NULL)
	bh2 -> ptr.free_ptr.next -> ptr.free_ptr.prev =
	  bh2  -> ptr.free_ptr.prev;
      
      if (bh2 -> ptr.free_ptr.prev != NULL)
	bh2 -> ptr.free_ptr.prev -> ptr.free_ptr.next =
	  bh2 -> ptr.free_ptr.next;
      
      fl = bh2 -> ptr.free_ptr.first_index;
      sl = bh2 -> ptr.free_ptr.second_index;
      
      /* bh2 must be deleted from fl_array */
      if (ptr_TLSF -> fl_array [fl].sl_array [sl] == bh2)
	ptr_TLSF -> fl_array [fl].sl_array [sl] = bh2 -> ptr.free_ptr.next;
      
      if (ptr_TLSF -> fl_array [fl].sl_array [sl] == NULL){
	TLSF__clear_bit (sl, ptr_TLSF -> fl_array[fl].bitmapSL);
	if (!ptr_TLSF -> fl_array[fl].bitmapSL) 
	  TLSF__clear_bit (fl, ptr_TLSF -> bitmapFL);
	
      }

      bh -> size += bh2 -> size + beg_header_overhead;
      if (!IS_LAST_BLOCK (bh2)) {
	bh3 = (block_header_t *) (__u8 *) (bh2 -> ptr.buffer + 
					   TLSF_WORDS2BYTES
					   (GET_BLOCK_SIZE(bh2)));
	bh3 -> prev_phys_block = bh;
      }
      
      if (IS_LAST_BLOCK (bh2)) SET_LAST_BLOCK (bh);
    }
  }

  /* is it free the previous physical block? */
  if (bh -> prev_phys_block != NULL) { // This block is not the first block
   
    bh2 = bh -> prev_phys_block;
  
    if (!IS_USED_BLOCK (bh2)) {

      if (bh2 -> ptr.free_ptr.next != NULL)
	bh2 -> ptr.free_ptr.next -> ptr.free_ptr.prev =
	  bh2  -> ptr.free_ptr.prev;
      
      if (bh2 -> ptr.free_ptr.prev != NULL)
	bh2 -> ptr.free_ptr.prev -> ptr.free_ptr.next =
	  bh2 -> ptr.free_ptr.next;
      
      fl = bh2 -> ptr.free_ptr.first_index;
      sl = bh2 -> ptr.free_ptr.second_index;
	
      if (ptr_TLSF -> fl_array [fl].sl_array [sl] == bh2)
	ptr_TLSF -> fl_array [fl].sl_array [sl] = bh2 -> ptr.free_ptr.next;
      
      if (ptr_TLSF -> fl_array[fl].sl_array[sl] == NULL){
	
	TLSF__clear_bit (sl, ptr_TLSF -> fl_array[fl].bitmapSL);
	if (!ptr_TLSF -> fl_array[fl].bitmapSL)
	  TLSF__clear_bit (fl, ptr_TLSF -> bitmapFL);
	
      }     
      bh2 -> size += bh -> size + beg_header_overhead;

      bh = bh2;
      if (!IS_LAST_BLOCK (bh)) {
	bh3 = (block_header_t *) (__u8 *) (bh -> ptr.buffer + 
					   TLSF_WORDS2BYTES
					   (GET_BLOCK_SIZE(bh)));
	bh3 -> prev_phys_block = bh;
      }
    
    }
 
  }

  /*
   * and now we can merge the free block with the initial memory
   */
  mapping_function (GET_BLOCK_SIZE (bh), &fl, &sl, ptr_TLSF);
  fl -= MIN_LOG2_SIZE;
  bh -> ptr.free_ptr.first_index = fl;
  bh -> ptr.free_ptr.second_index = sl;
  bh -> ptr.free_ptr.next = ptr_TLSF -> fl_array [fl].sl_array [sl];
  bh -> ptr.free_ptr.prev = NULL;
 
  if (ptr_TLSF -> fl_array [fl].sl_array [sl] != NULL)
    ptr_TLSF -> fl_array [fl].sl_array [sl] -> ptr.free_ptr.prev = bh;
  ptr_TLSF -> fl_array [fl].sl_array [sl] = bh;

  TLSF__set_bit (sl, ptr_TLSF -> fl_array[fl].bitmapSL);
  TLSF__set_bit (fl, ptr_TLSF -> bitmapFL);

  THREAD_UNLOCK(flags);
}

#else // USE_TLSF_MEMORY_ALLOCATOR

/* 
 * Symple dynamic memory algorithm
 */
extern void simple_dynamic_memory__free (void *ptr);

void free (void *ptr)
{
  simple_dynamic_memory__free (ptr);
}

#endif // USE_TLSF_MEMORY_ALLOCATOR
