/*----------------------------------------------------------------------------
 *-------------------------      M a R T E   O S      ------------------------
 *----------------------------------------------------------------------------
 *                                                             V1.57  Dic 2005
 *                               'r e a l l o c'
 *
 *                                      C
 *
 * File 'realloc.c'                                  By Miguel Masmano Tello
 *                                                      and MAR.
 *
 * Implements two versions of realloc:
 *
 * - Two Levels Segregate Fit memory allocator (TLSF): This algorithm
 *   allows memory allocation and deallocation with bounded response
 *   times (Developed at the Universidad Politcnica de Valencia by
 *   Miguel Masmano Tello <mmasmano@disca.upv.es>)
 *
 * - Simple algorithm: allows allocation of memory blocks with not
 *   deallocation. It is useful for applications that does not use
 *   dynamic memory at all or uses it only at initialization time.
 *
 * ---------------------------------------------------------------------------
 *
 * Two Levels Segregate Fit memory allocator (TLSF)
 * Version 1.1
 *
 * Written by Miguel Masmano Tello <mmasmano@disca.upv.es>
 * Copyright (C) Dec, 2002 OCERA Consortium
 * Release under the terms of the GNU General Public License Version 2
 *
 * ----------------------------------------------------------------------
 *  Copyright (C) 2004   Universidad de Cantabria, SPAIN
 *
 *  MaRTE OS web page: http://marte.unican.es
 *  Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
 *                     Michael González Harbour      mgh@unican.es
 *
 * MaRTE OS  is free software; you can  redistribute it and/or  modify it
 * under the terms of the GNU General Public License  as published by the
 * Free Software Foundation;  either  version 2, or (at  your option) any
 * later version.
 *
 * MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
 * WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 * MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 * General Public License for more details.
 *
 * You should have received  a  copy of  the  GNU General Public  License
 * distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
 * Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 * 02111-1307, USA.
 *
 * As a  special exception, if you  link this  unit  with other  files to
 * produce an   executable,   this unit  does  not  by  itself cause  the
 * resulting executable to be covered by the  GNU General Public License.
 * This exception does  not however invalidate  any other reasons why the
 * executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
#include <sys/marte_configuration_parameters.h>

#ifdef USE_TLSF_MEMORY_ALLOCATOR
/*
 * Two Levels Segregate Fit memory allocator (TLSF)
 * Version 1.1
 *
 * Written by Miguel Masmano Tello <mmasmano@disca.upv.es>
 * Copyright (C) Dec, 2002 OCERA Consortium
 * Release under the terms of the GNU General Public License Version 2
 *
 */
#include <malloc.h>
#include <string.h>
#include "tlsf.h"


/* see man realloc */
void *realloc (void *p, size_t new_len)
{
  __u8 *ptr_aux;
  block_header_t *b;

  if (p == NULL)
    return (void *) malloc (new_len);
  else if (new_len == 0) {
    free (p);
    return NULL;
  }

  ptr_aux = (__u8 *) malloc (new_len * sizeof (__u8));
  
  b = (block_header_t *) (((__u8 *) p) - 
			  TLSF_WORDS2BYTES(beg_header_overhead));

  memcpy ((__u8 *) ptr_aux, (__u8 *) p, TLSF_WORDS2BYTES(b -> size));

  free (p);

  return ((void *) ptr_aux);
}

#else // USE_TLSF_MEMORY_ALLOCATOR
#include <sys/types.h>

/*
 * Symple dynamic memory algorithm
 */
extern void *simple_dynamic_memory__realloc (void *p, size_t new_len);

void *realloc (void *p, size_t new_len)
{
  return simple_dynamic_memory__realloc (p, new_len);
}

#endif // USE_TLSF_MEMORY_ALLOCATOR
