/*----------------------------------------------------------------------------
 *-------------------------      M a R T E   O S      ------------------------
 *----------------------------------------------------------------------------
 *                                                             V1.57  Dic 2005
 *
 *         'T e x t   a n d   S e r i a l   C o n s o l e   D r i v e r'
 *
 *                                      C
 *
 * File 'text_and_serial_console.c'                                    by MAR.
 *
 * Driver that allows switching between serial and text consoles.
 *
 * ----------------------------------------------------------------------
 *  Copyright (C) 2004   Universidad de Cantabria, SPAIN
 *
 *  MaRTE OS web page: http://marte.unican.es
 *  Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
 *                     Michael González Harbour      mgh@unican.es
 *
 * MaRTE OS  is free software; you can  redistribute it and/or  modify it
 * under the terms of the GNU General Public License  as published by the
 * Free Software Foundation;  either  version 2, or (at  your option) any
 * later version.
 *
 * MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
 * WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 * MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 * General Public License for more details.
 *
 * You should have received  a  copy of  the  GNU General Public  License
 * distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
 * Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 * 02111-1307, USA.
 *
 * As a  special exception, if you  link this  unit  with other  files to
 * produce an   executable,   this unit  does  not  by  itself cause  the
 * resulting executable to be covered by the  GNU General Public License.
 * This exception does  not however invalidate  any other reasons why the
 * executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
#include <sys/types.h>
#include <stdio.h>

#include <drivers/text_and_serial_console.h>

/* Text Console Functions (defined in 'text_console.c') */
extern int text_console_create ();
extern int text_console_basic_create ();
extern void text_console_end_of_kernel_initialization ();
extern ssize_t text_console_write (int file_descriptor, void *buffer,
				   size_t bytes);
extern ssize_t text_console_write_error (int file_descriptor, void *buffer,
					 size_t bytes);
extern int  text_console_ioctl (int file_descriptor, int request, void *data);

/* Serial Port Functions (defined in 'serial_port_driver_c.c') */
extern int serial_port_init_console ();
extern void serial_port_end_of_kernel_initialization ();
extern ssize_t serial_port_console_write (int file_descriptor,
					  void *buffer, 
					  size_t bytes);
extern int serial_port_ioctl (int file_descriptor, int request, void* argp);

/*----------------------*
 *-- Global variables --*
 *----------------------*/
#define TEXT_CONSOLE   0
#define SERIAL_CONSOLE 1
static int current_console = TEXT_CONSOLE; // text console by default
struct {
  int (*create)();
  void (*end_of_kernel_initialization)();
  ssize_t (*write) (int, void *, size_t);
  ssize_t (*write_error) (int, void *, size_t);
  int (*ioctl) (int, int, void*);
} console_functions [2] = {{text_console_basic_create, 
			    text_console_end_of_kernel_initialization,
			    text_console_write, 
			    text_console_write_error, 
			    text_console_ioctl},
			   {serial_port_init_console,
			    serial_port_end_of_kernel_initialization,
			    serial_port_console_write,
			    serial_port_console_write,
			    serial_port_ioctl}};



/*---------------------------------------------------------------------------*
 *--  Text Console Driver Functions  ----------------------------------------*
 *---------------------------------------------------------------------------*/
// 'open', 'close', 'read' and 'remove' not implemented for this driver.

/*--------------*
 *--  Create  --*
 *--------------*/
int text_and_serial_console_create ()
{
  return console_functions[current_console].create ();
}

/*------------------------------------*
 *--  End of kernel initialization  --*
 *------------------------------------*/
void text_and_serial_console_end_of_kernel_initialization ()
{
  console_functions[current_console].end_of_kernel_initialization ();
}

/*-------------*
 *--  Write  --*
 *-------------*/
ssize_t text_and_serial_console_write (int file_descriptor, void *buffer,
				       size_t bytes)
{ 
  return console_functions[current_console].write (file_descriptor, buffer,
						   bytes);
}

/*-------------------*
 *--  Write Error  --*
 *-------------------*/
ssize_t text_and_serial_console_write_error (int file_descriptor, void *buffer,
					     size_t bytes)
{ 
  return console_functions[current_console].write_error (file_descriptor, 
							 buffer,
							 bytes);
}

/*-------------*
 *--  Ioctl  --*
 *-------------*/
int text_and_serial_console_ioctl (int file_descriptor, int request, 
				   void *data)
{
  switch (request) {
  case CHANGE_TO_TEXT_CONSOLE:
    console_functions[current_console].write 
      (file_descriptor, "\nConsole output sent to standard console...\n", 44);
    text_console_create ();
    current_console = TEXT_CONSOLE;
    break;

  case CHANGE_TO_SERIAL_CONSOLE:
    console_functions[current_console].write 
      (file_descriptor, "\nConsole output sent to serial port ...\n", 40);
    serial_port_init_console ();
    current_console = SERIAL_CONSOLE;
    console_functions[current_console].write 
      (file_descriptor, "------------------------------------------\n", 43);
    console_functions[current_console].write 
      (file_descriptor, "  MaRTE OS console output...\n", 29);

  default:
    return console_functions[current_console].ioctl (file_descriptor, request, 
						     data);
  }
  return 0;
}
