pragma Source_Reference (1, "hardware_interface.gpb");
------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--                   'H a r d w a r e _ I n t e r f a c e'
--
--                                  Body
--
--
--  File 'hardware_interface.adb'                                   By MAR.
--
--  PC Version.
--
--  This package is the border between the hardware dependent and hardware
--  independent parts of the kernel.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
-------------------------------------------------------------------------------
with System;
with Unchecked_Conversion;

with Processor_Registers;
pragma Elaborate_All (Processor_Registers);
with PIC;
pragma Elaborate_All (PIC);
with RTC;
pragma Elaborate_All (RTC);


with TSC; use TSC;
pragma Elaborate_All (TSC);






with PIT; use PIT;
pragma Elaborate_All (PIT);


package body Hardware_Interface is

   --------------------------------------------------------------------------
   -- Interrupts ------------------------------------------------------------
   --------------------------------------------------------------------------

   ----------------------------------
   -- Default_HW_Interrupt_Handler --
   ----------------------------------
   procedure Default_HW_Interrupt_Handler_C (State : in Trap_State_Ac);
   pragma Import (C, Default_HW_Interrupt_Handler_C,
                  "base_irq_default_handler");
   procedure Default_HW_Interrupt_Handler (State : in Trap_State_Ac) is
   begin
      Default_HW_Interrupt_Handler_C (State);
   end Default_HW_Interrupt_Handler;

   ----------------------------------
   -- Install_HW_Interrupt_Handler --
   ----------------------------------
   procedure Install_HW_Interrupt_Handler
     (Int_Num  : in HW_Interrupt;
      Handler  : in HW_Interrupt_Handler) is
   begin
      Interrupt_Tables.Install_IRQ_Handler
        (Interrupt_Tables.IRQ (Int_Num),
         Interrupt_Tables.IRQ_Handler_Type (Handler));
   end Install_HW_Interrupt_Handler;
   pragma Inline (Install_HW_Interrupt_Handler);

   ----------------------------------------------------
   -- Interrupt_Controller_Hardware_Enable_Interrupt --
   ----------------------------------------------------
   procedure Interrupt_Controller_Hardware_Enable_Interrupt
     (Int : in HW_Interrupt) is
   begin
      if Interrupt_Tables."/=" (Int,
                                Interrupt_Tables.LOCAL_APIC_TIMER_IRQ) then
         PIC.Enable_IRQ (Interrupt_Tables.IRQ (Int));
      end if;
   end Interrupt_Controller_Hardware_Enable_Interrupt;

   -----------------------------------------------------
   -- Interrupt_Controller_Hardware_Disable_Interrupt --
   -----------------------------------------------------
   procedure Interrupt_Controller_Hardware_Disable_Interrupt
     (Int : in HW_Interrupt) is
   begin
      if Interrupt_Tables."/=" (Int,
                                Interrupt_Tables.LOCAL_APIC_TIMER_IRQ) then
         PIC.Disable_IRQ (Interrupt_Tables.IRQ (Int));
      end if;
   end Interrupt_Controller_Hardware_Disable_Interrupt;

   -------------------------------------------
   -- Interrupt_Controller_Hardware_Enabled --
   -------------------------------------------
   procedure Interrupt_Controller_Hardware_Enabled is
   begin
      PIC.PIC_Master_Acknowledge_IRQ;
      PIC.PIC_Slave_Acknowledge_IRQ;
   end Interrupt_Controller_Hardware_Enabled;
   pragma Inline (Interrupt_Controller_Hardware_Enabled);


   --------------------------------------------------------------------------
   -- Time and Timers -------------------------------------------------------
   --------------------------------------------------------------------------













   --  'HWT_HZ' is the number of 'HWTime' units per second.

   HWT_HZ : constant HWTime := TSC.CPU_Hz;




   ---------------------------------
   -- Safe_Longest_Timer_Interval --
   ---------------------------------
   function Safe_Longest_Timer_Interval return HWTime is
   begin





      return (PIT.Safe_Longest_Timer_Interval * HWT_HZ) / PIT.PIT_HZ;

   end Safe_Longest_Timer_Interval;

   function To_HWT_NS is new Unchecked_Conversion (Duration, HWTime);
   function To_Duration is new Unchecked_Conversion (HWTime, Duration);
   NS_Per_S : constant HWTime := 10#1#E9;

   ------------------------
   -- HWTime_To_Duration --
   ------------------------
   function HWTime_To_Duration (Th : in HWTime) return Duration is
      S : HWTime; -- := Th;
   begin
      S := Th / HWT_HZ;
      return To_Duration (S * NS_Per_S +
                          ((Th - S * HWT_HZ) * NS_Per_S) /
                          HWT_HZ);
   end HWTime_To_Duration;
   pragma Inline (HWTime_To_Duration);

   ------------------------
   -- Duration_To_HWTime --
   ------------------------
   function Duration_To_HWTime (D : in Duration) return HWTime is
      S : HWTime; -- := To_HWT_NS (D);
   begin
      S  := To_HWT_NS (D) / NS_Per_S;
      return S * HWT_HZ +
             ((To_HWT_NS (D) - S * NS_Per_S) * HWT_HZ) / NS_Per_S;
   end Duration_To_HWTime;
   pragma Inline (Duration_To_HWTime);

























   --------------------------------------------------
   -- Program_Timer (TSC used to measure the time) --
   --------------------------------------------------
   --
   --  Should be called with interrupts disabled.
   procedure Program_Timer (Timer           : in  HW_Timers;
                            Interval        : in  HWTime;
                            Next_Activation : out HWTime) is
   begin

      --  Reprograms timer 0
      PIT.Write_Counter (PIT_CNT0,
                         Unsigned_16 ((Interval * PIT.PIT_HZ) / HWT_HZ));

      --  Gets next activation
      Next_Activation := TSC.Read_TSC + Interval;
   end Program_Timer;


















































































   ---------------------------
   -- Get_HWClock_Frequency --
   ---------------------------
   function Get_HWClock_Frequency return HWTime is
   begin
      return HWT_HZ;
   end Get_HWClock_Frequency;

   ------------------------------------
   -- Compulsory_Timer_Reprogramming --
   ------------------------------------
   function Compulsory_Timer_Reprogramming return Boolean is
   begin

      --  Time is measured using the TSC
      return False;





   end Compulsory_Timer_Reprogramming;

   ----------------------------
   -- RTC_HWTime_Since_Epoch --
   ----------------------------
   function RTC_HWTime_Since_Epoch return HWTime is
   begin
      return Duration_To_HWTime
        (Duration (RTC.RTC_Time_To_Seconds_Since_Epoch));
   end RTC_HWTime_Since_Epoch;

   -------------------------
   -- Finish_Hardware_Use --
   -------------------------
   procedure Finish_Hardware_Use is
   begin



      null;

   end Finish_Hardware_Use;


   -----------------------------------------------------------------------
   -- Inicialization -----------------------------------------------------
   -----------------------------------------------------------------------
   PIC_Master_IRQ_Base : constant := 16#20#;
   PIC_Slave_IRQ_Base  : constant := 16#28#;
   --  Any change in these values should be refected in
   --  'BASE_IRQ_MASTER_BASE' and 'BASE_IRQ_SLAVE_BASE' (file
   --  'include/oskit/x86/pc/base_irq.h').
begin
   pragma Assert (Duration'Small = 1.0/1e9);
   --  We rely that Duration is an exact count of nanoseconds.








   --  Configure interrupts.
   PIC.PICs_Init (Master_Base => PIC_Master_IRQ_Base,
                  Slave_Base  => PIC_Slave_IRQ_Base);
   Interrupt_Tables.Inicialize (Base_IDT => 16#0#);










   PIT.Inicialize (Cnt0_Ini => 16#FF00#, Cnt2_Ini => 0);
   --  Initial value for counter 0 big enough to avoid interrupt during
   --  kernel initialization. Counter 2 is not programmed, so it can be used
   --  for the speaker.






end Hardware_Interface;
