pragma Source_Reference (1, "hardware_interface.gps");
------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--                   'H a r d w a r e _ I n t e r f a c e'
--
--                                  Spec
--
--
--  File 'hardware_interface.ads'                                   By MAR.
--
--
--  This package is the border between the hardware dependent and hardware
--  independent parts of the kernel.
--
--  PC Version.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
-------------------------------------------------------------------------------
with System;
with Basic_Integer_Types; use Basic_Integer_Types;
with Interrupt_Tables;
pragma Elaborate_All (Interrupt_Tables);
with Processor_Registers;

with TSC;
pragma Elaborate_All (TSC);


package Hardware_Interface is

   pragma Elaborate_Body;

   ----------------
   -- Interrupts --
   ----------------
   subtype HW_Interrupt is Interrupt_Tables.IRQ;






   TIMER_INTERRUPT : constant HW_Interrupt := Interrupt_Tables.TIMER_IRQ;



   KEYBOARD_INTERRUPT    : constant HW_Interrupt :=
     Interrupt_Tables.KEYBOARD_IRQ;
   CTLR2_INTERRUPT       : constant HW_Interrupt :=
     Interrupt_Tables.CTLR2_IRQ;
   SERIAL2_INTERRUPT     : constant HW_Interrupt :=
     Interrupt_Tables.SERIAL2_IRQ;
   SERIAL1_INTERRUPT     : constant HW_Interrupt :=
     Interrupt_Tables.SERIAL1_IRQ;
   PARALLEL2_INTERRUPT   : constant HW_Interrupt :=
     Interrupt_Tables.PARALLEL2_IRQ;
   DISKETTE_INTERRUPT    : constant HW_Interrupt :=
     Interrupt_Tables.DISKETTE_IRQ;
   PARALLEL1_INTERRUPT   : constant HW_Interrupt :=
     Interrupt_Tables.PARALLEL1_IRQ;
   RTC_INTERRUPT         : constant HW_Interrupt :=
     Interrupt_Tables.RTC_IRQ;
   SOFT_INTERRUPT        : constant HW_Interrupt :=
     Interrupt_Tables.SOFT_IRQ;
   RESERVED1_INTERRUPT   : constant HW_Interrupt :=
     Interrupt_Tables.RESERVED1_IRQ;
   RESERVED2_INTERRUPT   : constant HW_Interrupt :=
     Interrupt_Tables.RESERVED2_IRQ;
   RESERVED3_INTERRUPT   : constant HW_Interrupt :=
     Interrupt_Tables.RESERVED3_IRQ;
   COPROCESSOR_INTERRUPT : constant HW_Interrupt :=
     Interrupt_Tables.COPROCESSOR_IRQ;
   FIXED_DISK_INTERRUPT  : constant HW_Interrupt :=
     Interrupt_Tables.FIXED_DISK_IRQ;
   RESERVED4_INTERRUPT   : constant HW_Interrupt :=
     Interrupt_Tables.RESERVED4_IRQ;

   subtype Trap_State is Interrupt_Tables.Trap_State; -- PC
   subtype Trap_State_Ac is Interrupt_Tables.Trap_State_Ac;

   subtype HW_Interrupt_Handler is
     Interrupt_Tables.IRQ_Handler_Type;

   procedure Default_HW_Interrupt_Handler (State : in Trap_State_Ac);

   procedure Install_HW_Interrupt_Handler
     (Int_Num  : in HW_Interrupt;
      Handler  : in HW_Interrupt_Handler);


   procedure Interrupts_Disabled renames Processor_Registers.CLI;

   procedure Interrupts_Enabled renames Processor_Registers.STI;

   procedure Interrupt_Controller_Hardware_Enable_Interrupt
     (Int : in HW_Interrupt);

   procedure Interrupt_Controller_Hardware_Disable_Interrupt
     (Int : in HW_Interrupt);

   procedure Interrupt_Controller_Hardware_Enabled;
   pragma Inline (Interrupt_Controller_Hardware_Enabled);
   --  Enable the hardware interrupt controller to generate interrupts for
   --  the Processor

   function Are_Interrupts_Enabled return Boolean
     renames Processor_Registers.Are_Interrupts_Enabled;
   pragma Inline (Are_Interrupts_Enabled);


   -----------------------------------------
   -- Save and Restore the Flags Register --
   -----------------------------------------
   procedure Save_Flags_And_Disable_Interrupts (EFlags : out Integer)
     renames Processor_Registers.Save_Flags_And_Disable_Interrupts;
   --  Exported to C with name "save_flags_and_disable_interrupts"

   function Save_Flags return Integer
     renames Processor_Registers.Save_Flags;

   procedure Restore_Flags (EFlags : in Integer)
     renames Processor_Registers.Restore_Flags;
   --  Exported to C  with name "restore_flags"

   ----------------------------
   -- Stack Pointer Register --
   ----------------------------
   function Get_Stack_Pointer_Register return Unsigned_32
     renames Processor_Registers.Get_ESP_Register;
   pragma Inline (Get_Stack_Pointer_Register);


   --------------------
   -- Bit Operations --
   --------------------
   procedure Bit_Set (Bit_Field : in out Unsigned_32; Bit : in Unsigned_32)
     renames Processor_Registers.Bit_Set;

   procedure Bit_Reset (Bit_Field : in out Unsigned_32; Bit : in Unsigned_32)
     renames Processor_Registers.Bit_Reset;

   procedure Bit_Scan_Forward (Bit_Field : in Unsigned_32;
                               Bit       : out Unsigned_32)
     renames Processor_Registers.Bit_Scan_Forward; -- 'Bit_Field' can't be 0!!

   procedure Bit_Scan_Reverse (Bit_Field : in Unsigned_32;
                               Bit       : out Unsigned_32)
     renames Processor_Registers.Bit_Scan_Reverse; -- 'Bit_Field' can't be 0!!

   ---------------------
   -- Time and Timers --
   ---------------------

   Last_HW_Timer : constant Integer_32 := 0; -- PC

   type HW_Timers is new Integer_32 range 0 .. Last_HW_Timer;
   for HW_Timers'Size use 32;

   HW_Timer_0 : constant HW_Timers := 0;



   subtype HWTime is TSC.Processor_Cycles;
   function "+" (A, B : HWTime) return HWTime renames TSC."+";
   function "-" (A, B : HWTime) return HWTime renames TSC."-";
   function "*" (A, B : HWTime) return HWTime renames TSC."*";
   function "/" (A, B : HWTime) return HWTime renames TSC."/";
   function "<" (A, B : HWTime) return Boolean renames TSC."<";
   function ">" (A, B : HWTime) return Boolean renames TSC.">";
   function "=" (A, B : HWTime) return Boolean renames TSC."=";

   function Get_HWTime_Slow return HWTime renames TSC.Read_TSC;

   function Get_HWTime return HWTime renames TSC.Read_TSC;














   function Safe_Longest_Timer_Interval return HWTime;
   --  The longest interval that can be used to program the timer.


   function HWTime_To_Duration (Th : in HWTime) return Duration;
   pragma Inline (HWTime_To_Duration);

   function Duration_To_HWTime (D : in Duration)  return HWTime;
   pragma Inline (Duration_To_HWTime);

   function Get_HWClock_Frequency return HWTime;
   --  Ticks per second
   pragma Inline (Get_HWClock_Frequency);

   procedure Program_Timer (Timer           : in HW_Timers;
                            Interval        : in HWTime;
                            Next_Activation : out HWTime);
   pragma Inline (Program_Timer);

   function Compulsory_Timer_Reprogramming return Boolean;
   --  Returns true if the timer should always be reprogrammed even if
   --  there isn't any event. This could happen, for example, when the
   --  timer interrupts are also used for the system clock.

   ---------------------
   -- Real Time Clock --
   ---------------------
   RTC_Available : constant Boolean := True;
   function RTC_HWTime_Since_Epoch return HWTime;

   -------------------
   -- Context Swich --
   -------------------
   procedure Context_Switch (Old_Task : in System.Address;
                             New_Task : in System.Address)
     renames Processor_Registers.Context_Switch;
   pragma Inline (Context_Switch);

   procedure Change_Return_Address_Of_Preempted_Task
     (Top_Of_Stack    : in Unsigned_32;
      New_Ret_Address : in System.Address)
     renames Processor_Registers.Change_Return_Address_Of_Preempted_Task;

   -------------------------
   -- Finish_Hardware_Use --
   -------------------------
   procedure Finish_Hardware_Use;
   --  Invoked while finishing the application to leave the hardware used
   --  by MaRTE OS in a known and stable status. Called from 'our_exit' in
   --  'base_console_init.c'.



end Hardware_Interface;
