------------------------------------------------------------------------------
-- -------------------         M a R T E   O S         -------------------- --
------------------------------------------------------------------------------
--                                                             V1.57  Dic 2005
--
--                        'I O _ I n t e r f a c e'
--
--                                 Body
--
--
--  File 'io_interface.adb'                                            By Mar.
--
--
--  Operations for accessing I/O PC space.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2004   Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael González Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------

with System.Machine_Code; use System.Machine_Code;

package body IO_Interface is

   procedure IOdelay;
   procedure IOdelay is
   begin
      --  Using outb's to a nonexistent port seems to guarantee better
      --  timings even on fast machines. (Linux).

      Asm ("outb %%al,$0x80", No_Output_Operands, No_Input_Operands);


   end IOdelay;
   pragma Inline (IOdelay);

   ---------
   -- Inb --
   ---------
   function Inb (Port : in IO_Port) return Unsigned_8 is
      Tmp : Unsigned_8;
   begin
      Asm ("inb %%dx, %0",
           Unsigned_8'Asm_Output ("=a", Tmp),
           IO_Port'Asm_Input  ("d",  Port),
           "",
           True);
      return Tmp;
   end Inb;
   pragma Inline (Inb);

   -----------
   -- Inb_P --
   -----------
   function Inb_P (Port : in IO_Port) return Unsigned_8 is
      Tmp : Unsigned_8;
   begin
      Tmp := Inb (Port);
      IOdelay;
      return Tmp;
   end Inb_P;
   pragma Inline (Inb_P);

   ---------
   -- Inw --
   ---------
   function Inw (Port : in IO_Port) return Unsigned_16 is
      Tmp : Unsigned_16;
   begin
      Asm ("inw %%dx, %0",
           Unsigned_16'Asm_Output ("=a", Tmp),
           IO_Port'Asm_Input  ("d",  Port),
           "",
           True);
      return Tmp;
   end Inw;
   pragma Inline (Inw);

   -----------
   -- Inw_P --
   -----------
   function Inw_P (Port : in IO_Port) return Unsigned_16 is
      Tmp : Unsigned_16;
   begin
      Tmp := Inw (Port);
      IOdelay;
      return Tmp;
   end Inw_P;
   pragma Inline (Inw_P);

   ---------
   -- Inl --
   ---------
   function Inl (Port : in IO_Port) return Unsigned_32 is
      Tmp : Unsigned_32;
   begin
      Asm ("inl %%dx, %0",
           Unsigned_32'Asm_Output ("=a", Tmp),
           IO_Port'Asm_Input  ("d",  Port),
           "",
           True);
      return Tmp;
   end Inl;
   pragma Inline (Inl);

   -----------
   -- Inl_P --
   -----------
   function Inl_P (Port : in IO_Port) return Unsigned_32 is
      Tmp : Unsigned_32;
   begin
      Tmp := Inl (Port);
      IOdelay;
      return Tmp;
   end Inl_P;
   pragma Inline (Inl_P);

   ----------
   -- Outb --
   ----------
   procedure Outb (Port : in IO_Port; Val : in Unsigned_8) is
   begin
      Asm ("outb %0, %%dx",
           No_Output_Operands,
           (Unsigned_8'Asm_Input ("a", Val),
            IO_Port'Asm_Input ("d", Port)),
           "", True);
   end Outb;
   pragma Inline (Outb);

   ------------
   -- Outb_P --
   ------------
   procedure Outb_P (Port : in IO_Port; Val : in Unsigned_8) is
   begin
      Outb (Port, Val);
      IOdelay;
   end Outb_P;
   pragma Inline (Outb_P);

   ----------
   -- Outw --
   ----------
   procedure Outw (Port : in IO_Port; Val : in Unsigned_16) is
   begin
      Asm (".byte 0x66; outl %0, %%dx",
           No_Output_Operands,
           (Unsigned_16'Asm_Input ("a", Val),
            IO_Port'Asm_Input ("d", Port)),
           "", True);
   end Outw;
   pragma Inline (Outw);

   ------------
   -- Outw_P --
   ------------
   procedure Outw_P (Port : in IO_Port; Val : in Unsigned_16) is
   begin
      Outw (Port, Val);
      IOdelay;
   end Outw_P;
   pragma Inline (Outw_P);

   ----------
   -- Outl --
   ----------
   procedure Outl (Port : in IO_Port; Val : in Unsigned_32) is
   begin
      Asm ("outl %0, %%dx",
           No_Output_Operands,
           (Unsigned_32'Asm_Input ("a", Val),
            IO_Port'Asm_Input ("d", Port)),
           "", True);
   end Outl;
   pragma Inline (Outl);

   ------------
   -- Outl_P --
   ------------
   procedure Outl_P (Port : in IO_Port; Val : in Unsigned_32) is
   begin
      Outl (Port, Val);
      IOdelay;
   end Outl_P;
   pragma Inline (Outl_P);

end IO_Interface;
